#ifndef KEP_TOOLBOX_CONFIG_HPP
#define KEP_TOOLBOX_CONFIG_HPP

// Start of defines instantiated by CMake.
// clang-format off
#define KEP_TOOLBOX_VERSION_STRING "2.4.1"
#define KEP_TOOLBOX_VERSION_MAJOR 2
#define KEP_TOOLBOX_VERSION_MINOR 4
#define PYKEP_USING_SPICE


#endif
